// ignore_for_file: cancel_subscriptions, public_member_api_docs
// ignore_for_file: non_constant_identifier_names, empty_catches
// ignore_for_file: avoid_dynamic_calls

import 'dart:async';

import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart';
import 'package:healthrian_common_support/'
    'healthrian_common_support.dart';
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as wearecg12;

class BLEManager with Ecg {
  factory BLEManager() {
    return _instance;
  }

  BLEManager._internal() {}

  static final BLEManager _instance = BLEManager._internal();

  BluetoothDevice? device;

  Versions version = Versions.v1;
  late final wearecg12.BLEManager _v1 = wearecg12.BLEManager()..scanFilter = _scanFilter;
  final BLEManagerV2 _v2 = BLEManagerV2();

  final streams = Streams();

  bool Function(BluetoothDevice device) _scanFilter =
      (device) => device.name.isCardioClick;

  bool Function(BluetoothDevice device) get scanFilter => _scanFilter;
  set scanFilter (bool Function(BluetoothDevice device) value) {
    _scanFilter = value;

    switch (version){
      case Versions.v1:
        _v1.scanFilter = value;
        break;
      case Versions.v2:
        _v2.scanFilter = value;
        break;
    }
  }

  Future<bool> connectDevice({String preferredDeviceId = ''}) async {
    switch (version){
      case Versions.v1:
        return await _v1.connectDevice(preferredDeviceId: preferredDeviceId);
      case Versions.v2:
        return await _v2.connectDevice(preferredDeviceId: preferredDeviceId);
    }
  }

  /// [disconnectDevice] disconnects all devices
  /// whose name start with `C-Click`
  Future<void> disconnectDevice() async {
    switch (version){
      case Versions.v1:
        await _v1.disconnectDevice();
        break;
      case Versions.v2:
        await _v2.disconnectDevice();
        break;
    }
  }

  /// [registerCharacteristics] registers all characteristics,
  /// defined in [Characteristics].
  ///
  /// The result is registered in a map structure [points].
  Future<void> registerCharacteristics() async {
    switch (version){
      case Versions.v1:
        await _v1.registerCharacteristics();
        break;
      case Versions.v2:
        await _v1.registerCharacteristics();
        break;
    }
  }

  /// [initializeStreams] is used to reset measurement.
  ///
  /// Once calling it,
  /// all measurement streams become unobservable.
  Future<void> initializeStreams() async {
    switch (version){
      case Versions.v1:
        await _v1.initializeStreams();
        break;
      case Versions.v2:
        await _v2.initializeStreams();
        break;
    }
  }
}
